VERSION = (0, 0, 1, "a", 33)  # following PEP 386

def get_version():
    version = "%s.%s" % (VERSION[0], VERSION[1])
    if VERSION[2]:
        version = "%s.%s" % (version, VERSION[2])
    if VERSION[3] != "f":
        version = "%s%s%s" % (version, VERSION[3], VERSION[4])
#        if DEV_N:
#            version = "%s.dev%s" % (version, DEV_N)
    return version

__version__ = get_version()



import datetime
from haystack import indexes
from models import Article
# from guardian.shortcuts import get_groups_with_perms

from django.contrib.auth.models import Group
everybody_group = Group.objects.get(name="Everybody")

class ArticleIndex(indexes.SearchIndex, indexes.Indexable):
    text        = indexes.CharField(document=True, use_template=True)
    author      = indexes.CharField(model_attr='user')
    created     = indexes.DateTimeField(model_attr='created')
    group_access= indexes.MultiValueField()

    def get_model(self):
        return Article

    def index_queryset(self, using=None):
        return self.get_model().objects.filter(created__lte=datetime.datetime.now())

    def get_updated_field(self):
        "created"

    def prepare_group_access(self, obj):
        return [everybody_group.pk]
        allowed_groups = []
        groups = Group.objects.filter(name='Everybody')
        for group in groups:
            allowed_groups.append(group.id)
        return allowed_groups



from django.core.management.base import BaseCommand, CommandError
import logging
logging.basicConfig(filename='calculate_scores.log', level=logging.DEBUG, format='%(asctime)s %(message)s')

from djig.models import Article
from datetime import datetime

class Command(BaseCommand):
    help = "Loops through articles and calculates score."

    def handle(self, *args, **options):
        logging.debug("Calculating scores at %s" % datetime.now())
        articles = Article.objects.all()
        for article in articles:
            article.calculate_score()
            logging.debug("Calculated score for %s" % (article.title))
        logging.debug("Finished calculating scores")
        self.stdout.write("Finished calculating scores at %s." % datetime.now())



from django.db import models
from django.contrib.auth.models import User, Group
from django.forms import ModelForm
import datetime
from uuslug import uuslug 
import times
from django.utils.encoding import smart_unicode
from math import ceil
from guardian.shortcuts import assign, get_perms, get_groups_with_perms

everybody_group = Group.objects.get(name="Everybody")

class Category(models.Model):
    title                   = models.CharField(max_length=100)
    slug                    = models.CharField(max_length=100, null=True, blank=True)
    image                   = models.URLField(null=True, blank=True)
    description             = models.TextField()

    class Meta:
        verbose_name_plural = "Categories"

    def __unicode__(self):
        return self.title

class Article(models.Model):
    title                   = models.CharField(max_length=100)
    slug                    = models.CharField(max_length=100, null=True, blank=True)
    url                     = models.URLField(null=True, blank=True)
    site                    = models.URLField(null=True, blank=True)
    site_name               = models.CharField(max_length=50, null=True, blank=True)
    self                    = models.NullBooleanField(default=False)
    description             = models.TextField(null=True, blank=True)
    image                   = models.URLField(null=True, blank=True)
    user                    = models.ForeignKey(User)
    category                = models.ForeignKey(Category, null=True, blank=True)
    created                 = models.DateTimeField(null=True, blank=True)
    love_count              = models.IntegerField(default=0)
    read_count              = models.IntegerField(default=0)
    comment_count           = models.IntegerField(default=0)
    calculated_score        = models.DecimalField(max_digits=10, decimal_places=10, default=0, null=True, blank=True)

    def __unicode__(self):
        return self.title

    class Meta:
        permissions = ( 
            ('group_view', 'Groups that can view'),
        ) 

    def save(self, *args, **kwargs):
        if not self.id:
            self.slug = uuslug(self.title, instance=self)
            self.title = self.title.encode("ascii", "ignore")
            self.created = datetime.datetime.now()
            assign('djig.group_view', everybody_group, self)
        super(Article, self).save(*args, **kwargs)

    @models.permalink
    def get_absolute_url(self):
        return ('djig_article_detail', [self.slug])

    def calculate_score(self):
        now     = times.now()
        then    = times.to_universal(self.created)
        hour_age= ceil((now-then).total_seconds()/60/60)
        gravity = 1.8
        self.calculated_score = (self.love_count-1)/pow((hour_age+2), gravity)
        self.save()
        print "%s -- %s -- %s" % (self.calculated_score, hour_age, self.title)
        return (self.love_count-1) / pow((hour_age+2), gravity)

    def get_allowed_groups(self):
        allowed_groups = []
        groups = get_groups_with_perms(self, attach_perms=True)
        for group in groups.keys():
            if "group_view" in groups[group]:
                allowed_groups.append(group)
        return allowed_groups

class ArticleFormLean(ModelForm):
    class Meta:
        model = Article
        exclude = ('title', 'slug', 'site', 'site_name', 'self', 'description', 'image', 'user', 'category', 
        'created', 'love_count', 'read_count', 'comment_count')

class ArticleForm(ModelForm):
    class Meta:
        model = Article
        exclude = ('slug', 'self', 'image', 'user', 'category', 'created', 'love_count',
        'read_count', 'comment_count')



import logging
logger = logging.getLogger('djig')

from django.core.signals import request_finished
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver

from models import Article
import datetime
from django.dispatch.dispatcher import Signal
from voting.models import *
from django.contrib.contenttypes.models import ContentType
from django.db.models import F
from django.contrib.comments.signals import comment_was_posted
from django.contrib.comments.models import Comment

CONTENT_TYPE = ContentType.objects.get(app_label='djig', name='article')

# Listen for article saves, adjust love_score accordingly.
@receiver(post_save, sender=Article, dispatch_uid='listen_to_article')
def listen_to_article(sender, instance, created, **kwargs):
    if created:
        logger.debug("DJIG_SIGNALS: New article created")
    else:
        logger.debug("DJIG_SIGNALS: Article was edited.")

@receiver(post_save, sender=Vote, dispatch_uid='listen_to_votes')
def listen_to_vote(sender, instance, created, **kwargs):
    if created:
        if instance.content_type == CONTENT_TYPE:
            # This vote is being applied to a djig article.
            article = Article.objects.get(pk=instance.object_id)
            logger.debug(article.love_count)
            # We don't want the initial vote to count for too much.
            same_user = instance.user == article.user
            if same_user:
                Article.objects.filter(pk=instance.object_id).update(love_count=F('love_count') + 1)
                logger.debug("Submission user vote - incremented by 1.")
            else:
                Article.objects.filter(pk=instance.object_id).update(love_count=F('love_count') + 2)
                logger.debug("Different user vote - incremented by 2.")
    else:
        logger.debug("DJIG_SIGNALS: Vote was edited or rescinded")

@receiver(comment_was_posted, sender=Comment, dispatch_uid='listen_to_comments')
def listen_to_comment(sender, comment, request, **kwargs):
    if comment.content_type == CONTENT_TYPE:
        logger.debug("This is a new comment on a Djig article")
        article = Article.objects.get(pk=comment.object_pk)
        same_user = comment.user == article.user
        if same_user:
            Article.objects.filter(pk=comment.object_pk).update(love_count=F('love_count') + 1)
            logger.debug("Comment by same user - incremented by 1.")
        else:
            Article.objects.filter(pk=comment.object_pk).update(love_count=F('love_count') + 2)
            logger.debug("Comment by non-submission user - incremented by 2.")




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Category'
        db.create_table('djig_category', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('image', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('djig', ['Category'])

        # Adding model 'Article'
        db.create_table('djig_article', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('site', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('site_name', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('self', self.gf('django.db.models.fields.NullBooleanField')(default=False, null=True, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('image', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['djig.Category'], null=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('love_count', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('read_count', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('comment_count', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('calculated_score', self.gf('django.db.models.fields.DecimalField')(default=0, null=True, max_digits=10, decimal_places=10, blank=True)),
        ))
        db.send_create_signal('djig', ['Article'])


    def backwards(self, orm):
        # Deleting model 'Category'
        db.delete_table('djig_category')

        # Deleting model 'Article'
        db.delete_table('djig_article')


    models = {
        'actstream.action': {
            'Meta': {'ordering': "('-timestamp',)", 'object_name': 'Action'},
            'action_object_content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'action_object'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'action_object_object_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'actor_content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'actor'", 'to': "orm['contenttypes.ContentType']"}),
            'actor_object_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'target_content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'target'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'target_object_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'verb': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'djig.article': {
            'Meta': {'object_name': 'Article'},
            'calculated_score': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '10', 'decimal_places': '10', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['djig.Category']", 'null': 'True', 'blank': 'True'}),
            'comment_count': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'love_count': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'read_count': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'self': ('django.db.models.fields.NullBooleanField', [], {'default': 'False', 'null': 'True', 'blank': 'True'}),
            'site': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'site_name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'djig.category': {
            'Meta': {'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['djig']


from django.contrib import sitemaps
import datetime
from djig.models import Article

class DjigSitemap(sitemaps.Sitemap):    
    changefreq = "daily"    
    priority   = 0.5    
    
    def items(self):        
        return Article.objects.all()    
    
    def lastmod(self, obj):
        return obj.created



from django.conf.urls import patterns, include, url
from voting.views import vote_on_object

from tastypie.api import Api
from api import *
import signals

v1_api = Api(api_name='v1')
v1_api.register(NewsResource())

urlpatterns = patterns('',
    url(r'^$',                                          'djig.views.index',             name='djig_home'),
    url(r'^newest/$',                                   'djig.views.newest',            name='djig_newest'),
    url(r'^submit_link/$',                              'djig.views.submit_link',       name='djig_submit_link'),
    url(r'^submit_link_detail/$',                       'djig.views.submit_link_detail',name='djig_submit_link_detail'),
    url(r'^like/(?P<article_id>\d+)/$',                 'djig.views.like_article',      name='djig_like_article'),
    url(r'^article/(?P<slug>[-\w]+)/$',                 'djig.views.article_detail',    name='djig_article_detail'),
)



from django.http import HttpResponse
from django.shortcuts import render, redirect, get_object_or_404
import requests
from BeautifulSoup import BeautifulSoup
from models import Article, ArticleForm, ArticleFormLean
from urlparse import urlparse
from django.contrib.auth.decorators import login_required
from voting.models import Vote
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from datetime import date, timedelta
from text_utils import unescape
from voting.models import Vote
from django.views.decorators.csrf import csrf_exempt
from guardian.shortcuts import assign, remove_perm
from django.contrib.auth.models import Group, User

everybody_group = Group.objects.get(name="Everybody")

from django.conf import settings
if "actstream" in settings.INSTALLED_APPS:
    from actstream import action

COMMENTS_ENABLED = False
if "django.contrib.comments" in settings.INSTALLED_APPS:
    COMMENTS_ENABLED = True

def index(request):
    time_limit = date.today() - timedelta(hours=300)
    articleset = Article.objects.filter().order_by('-calculated_score')[:100]
    page = request.GET.get('page')
    paginator = Paginator(articleset, 10)

    try:
        articles = paginator.page(page)
    except PageNotAnInteger:
        articles = paginator.page(1)
    except EmptyPage:
        articles = paginator.page(paginator.num_pages)

    return render(request, "djig/index.html", {
        "articles"          : articles,
        "paginator"         : paginator,
        "page"              : page,
        "COMMENTS_ENABLED"  : COMMENTS_ENABLED,
    })

def newest(request):
    time_limit = date.today() - timedelta(hours=300)
    articleset = Article.objects.filter().order_by('-created')[:100]
    page = request.GET.get('page')
    paginator = Paginator(articleset, 10)

    try:
        articles = paginator.page(page)
    except PageNotAnInteger:
        articles = paginator.page(1)
    except EmptyPage:
        articles = paginator.page(paginator.num_pages)

    return render(request, "djig/index.html", {
        "articles"          : articles,
        "paginator"         : paginator,
        "page"              : page,
        "COMMENTS_ENABLED"  : COMMENTS_ENABLED,
    })

def article_detail(request, slug):
    article = get_object_or_404(Article, slug=slug)
    return render(request, "djig/detail.html", {
        "article"           : article,
        "COMMENTS_ENABLED"  : COMMENTS_ENABLED,
    })

@login_required
def submit_link(request):
    if request.method == "POST":
        form = ArticleFormLean(request.POST)
        if form.is_valid():
            url  = form.cleaned_data["url"]
            existing_article = Article.objects.filter(url=url)
            if existing_article.count() > 0:
                return HttpResponse("Duplicate")
            else:
                r = requests.get(url)
                soup = BeautifulSoup(r.text)
                title = unescape(soup.title.string)

                o = urlparse(url)
                site = "%s://%s" % (o.scheme, o.netloc)
                site_name = "%s" % o.netloc.replace("www.", "")
                data = {
                    'url': url, 
                    'title': title,
                    'site': site,
                    'site_name': site_name,
                }
                form = ArticleForm(initial=data)
                return render(request, "djig/submit_link.html", {
                    "form"          : form,
                })
        else:
            return HttpResponse(form.errors)
    else:
        form = ArticleForm()
        return render(request, "djig/submit_link.html", {
            "form"          : form,
        })

@login_required
def submit_link_detail(request):
    if request.method == "POST":
        form = ArticleForm(request.POST)
        if form.is_valid():
            obj = form.save(commit=False)
            obj.user = request.user
            obj.save()
            Vote.objects.record_vote(obj, request.user, +1)
            if "actstream" in settings.INSTALLED_APPS:
                action.send(request.user, verb='submitted', target=obj)
            return redirect("/")
        else:
            form = ArticleForm(request.POST)
            return render(request, "djig/submit_link.html", {
                "form"          : form,
            })
    else:
        return HttpResponse("You should POST to this page, not GET it.")
            

@csrf_exempt
@login_required
def like_article(request, article_id):
    article = get_object_or_404(Article, pk=article_id)
    Vote.objects.record_vote(article, request.user, +1)
    # article.love_count = article.love_count + 1
    # article.save()
    # article.calculate_score()
    if "actstream" in settings.INSTALLED_APPS:
        action.send(request.user, verb='liked', target=article)
    if request.method == "POST":
        return HttpResponse("{'result': 'success'}")
    else: 
        redirect_var = request.GET.get('next')
        if redirect_var:
            print redirect_var
            return redirect(redirect_var)
        else:
            return redirect("/")
